@@ -40,6 +40,8 @@ COMMANDS = dict(
     SSH_AGENTC_ADD_RSA_ID_CONSTRAINED=24,
     SSH2_AGENTC_ADD_ID_CONSTRAINED=25,
     SSH_AGENTC_ADD_SMARTCARD_KEY_CONSTRAINED=26,
+    SSH_AGENTC_EXTENSION=27,
+    SSH_AGENT_EXTENSION_FAILURE=28,
 )
 
 
@@ -86,6 +88,7 @@ class Handler:
             msg_code('SSH_AGENTC_REQUEST_RSA_IDENTITIES'): _legacy_pubs,
             msg_code('SSH2_AGENTC_REQUEST_IDENTITIES'): self.list_pubs,
             msg_code('SSH2_AGENTC_SIGN_REQUEST'): self.sign_message,
+            msg_code('SSH_AGENTC_EXTENSION'): _unsupported_extension,
         }
 
     def handle(self, msg):
@@ -144,6 +147,10 @@ class Handler:
             signature = self.conn.sign(blob=blob, identity=key['identity'])
         except IOError:
             return failure()
+        except Exception:
+            log.exception('signature with "%s" key failed', label)
+            raise
+
         log.debug('signature: %r', signature)
 
         try:
@@ -158,3 +165,8 @@ class Handler:
         data = util.frame(util.frame(key['type']), util.frame(sig_bytes))
         code = util.pack('B', msg_code('SSH2_AGENT_SIGN_RESPONSE'))
         return util.frame(code, data)
+
+
+def _unsupported_extension(buf):
+    code = util.pack('B', msg_code('SSH_AGENT_EXTENSION_FAILURE'))
+    return util.frame(code)
